
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The function of this script:
%
% (1) Calculate NF of non-inverting OPA based on Rg and Rf (in this case, LMH6629. 
% Other OPA can be simulated by replacing the en and ei in sub-function
% NF_Calculation_LMH6629_NonInverting with different spec).
%
% (2) Calculate stability map of OPA based on measured S-para matrix.
%
% (3) Simulate a arbitrary matching network between coil and OPA, and calcualte the
% matched impedances for both side.
%
% (4) Calculate the OPA-NF and decoupling for an arbitrary setup of coil/MN/OPA. 

% Author:
% Chenhao Sun,   PhD Student, Texas A&M Magnetic Resonance Systems Lab
% Further instructions are in the included file: 
%    Instructions_HighZ_Preamp_Analysis.PDF 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
close all
clc
tic % timer

% Part 1. Constant & initial Setup

%% setup gammas for smith chart. gammas is a matrix of admittance that
% uniformly disturbited across the smith chart. gammas will be used in
% the calculation that give results are based on different points on 
% smith chart. 
markersize = 4; %4 size of each dot on Smith chart
Cres = 256; % numbers of color scale
Nrings = 36; % number of concentric rings on Smith Chart
dfactor = 300; % bigger the number, denser the dots on smith chart
% Admittance Matrix uniformly disturbuted around Smith map
Gammas = Gamma_Setup(Nrings,dfactor); 


%% setup the resistors value for the non-inverting type OPA
Rf = 200;
Rg = 10;
InputCap = 7e-12;
%% Input Coil/OPA Impedance, measured or simulated. 
% the impedance of coil and OPA is measured using HP4195 between
% 10MHz-220MHz with 401 points.
%
% Different Impedance can be applied here.

% % input measured OPA impedance
data = load('Z_opa.mat').Final;
Freq = data(:,1);
Z_opa = data(:,2);

% % input measured Coil impedance
data = load('Z_coil.mat').Final;
Freq = data(:,1);
Z_coil = data(:,2);

% frequency index is specified here for bandwidth between 10-220MHz
% Assuming at 4.7T 

Freq_30MHZ_Index =  139; %2H 30.7192 MHZ
Freq_53MHZ_Index =  287;  %23Na

Data(1:401,1) = 1; 

% Show coil and OPA impedance 
figure
SmithPlot2(Z_opa,Data,256,0,0,"None",3, "NF_Area_Off", "Nuclei_On",0 );
title({'Measured Input Impedance of OPA',' '})
figure
SmithPlot2(Z_coil,Data,256,0,0,"None",3, "NF_Area_Off", "Nuclei_On",0 );
title({'Measured Coil Raw Impedance',' '})


%% Input Sparameter Matrix and calcualte stability circle
% S-parameter measured by VNA4195A, 10-220MHz. 
% S-para have be measured in linear |Gamma|/Phase mode#2,

% Read S para matrix
[Raw_S21,S11,S22,S21,S12] = Read_Sparameter('PwrOn');
%Calculate Stability Circle, ploted inside this function
[CL,RL,CS,RS] = Stability_Circle(Freq,markersize,Cres,Gammas,S11,S22,S21,S12,Freq_53MHZ_Index);


%% Define MN Matrix 

%Matching Network's matrix is defined here, every type is independent and 
%can be cascaded. 

%  Unit   |     Type     |  Config  |  addtional values  |      
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   1st   |      2nd     |   3rd    |   4th    |   5th   |   
%   H     |      0-L     | 0-Series |    0     |    0    |
%   F     |      1-C     | 1-Shunt  |    0     |    0    |
% Z Ratio |     2-Tsfr   |  2-Tsfr  |    0     |    0    |
%   Z0    |      3-TL    |   3-TL   | Length(m)|    0    |
%   Ohm   |      4-R     |    0     |    0     |    0    |
%   C1-F  |  5-LCC_type1 |    0     |    C2-F  |    L-H  |
%   C1-F  |  6-LCC_Type2 |    0     |    C2-F  |    L-H  |
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% Example %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  [ seriec C(5pF) -> Ideal Transformer(1:4) -> shunt C(10pF) ]
% can be defined as:
% Components_Matrix = [ 
%                       [ 5e-12  1  0  0  0];
%                       [ 4      2  2  0  0];
%                       [ 10e-12 1  1  0  0];
%                        ];

% Regular 50 MN for 200MHz two caps
% Element_1 = [ 4.99e-12 1 0 0]; %series C
% Element_2 = [ 33.62e-12 1 1 0]; %shunt C
% Components_Matrix = [ 
%                       Element_1;
%                       Element_2;
%                        ];

% Regular 50 MN for 200MHz #2 with three caps
% Element_1 = [ 5.37e-12 1 0 0]; %series C
% Element_2 = [ 15.98e-12 1 1 0]; %shunt C
% Element_3 = [ 14.4e-12 1 0 0]; %series C
% Components_Matrix = [ 
%                       Element_1;
%                       Element_2;
%                       Element_3;
%                        ];  

%% Regular 50 MN for 53MHz 
% Element_1 = [ 104.5e-12 1 0 0]; %series C
% Element_2 = [ 845e-12 1 1 0]; %shunt C
% Components_Matrix = [ 
%                       Element_1;
%                       Element_2;
%                        ];
   
%% Trying Resistor, may not be a good idea. 

LCC_1 = [ 59.3e-12 5 0 181e-12 81e-9]; %series C
Tsfr = [15 2 2 0 0];

Components_Matrix = [ 
                      LCC_1;
                      Tsfr;
                       ];  
              

%% Compute the matched impedance toward both side 

[Z_right,Z_left,VI2] = AfterMatching( Freq, Z_coil  , Z_opa ,Components_Matrix);
                  
figure %smith plot need gamma to be horizontally matrix
SmithPlot2(Z_right,Data,256,0,0,"None",3,"NF_Area_OFF", "Nuclei_On",0);
title({ 'Z right, matched impedance that OPA see', ' '})

% figure %smith plot need gamma to be horizontally matrix
% SmithPlot2(Z_left,Data,256,0,0,"None",3,"NF_Area_Off", "Nuclei_On",0);
% title({ 'Z left, impedance Coil that see', ' '})

V2(:,1) = abs( reshape( VI2(1,1,:) , [401,1,1]) );
% figure %smith plot need gamma to be horizontally matrix
% SmithPlot2(Z_right,Data,256,0,0,"None",3,"NF_Area_On", "Nuclei_On_Voltage",V2);
% title({ 'Voltage induced on OPA', ' '})




%% Part 2.2 Calculate NF Considering input capacitance
% % calculate OPA's NF, adding the input parastic caps of ~4.8pF
% % Simply by paralling the orgianl source Z with a shunt cap 4.8pF, and
% % transform to a new equivelent Z that OPA sees. 
% 

Z_original = gamma2z(Gammas); % Original source Z for OPA
Z_Cap = 1/(1i*2*pi*Freq(Freq_30MHZ_Index)*InputCap); %input parasitc cap's impedance
R_withCap = 1 ./ ( (1./Z_original) + (1./Z_Cap) ); 
%this is the equivelent impedance, paralleled from orignal source Z and input
%cap
Rs = real( R_withCap ); %take the real part, only resistive part
[NFs_1H] = NF_Calculation_LMH6629_NonInverting(Rf,Rg,Rs);

Dmax = 3;% plot on smith
Dmin = 0;
figure
SmithPlot(Gammas,NFs_1H,Cres,Dmax,Dmin,"Higher",markersize)
title({ ['2H OPA-NF vs Matched-to impedance, threshold at ', num2str(Dmax), ' dB'], ' ' })
colorbar('ticks', [0,0.25,0.5,0.75,1], 'TickLabels', ...
    { [ num2str(min(NFs_1H)) 'dB' ],  [ num2str( (min(NFs_1H)+((min(NFs_1H)+Dmax)/2))/2 ) 'dB' ],...
    [ num2str( ((min(NFs_1H)+Dmax)/2) ) 'dB'],[ num2str( (((min(NFs_1H)+Dmax)/2)+Dmax)/2 ) 'dB' ],...
    ['>' num2str(Dmax) 'dB'] })

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Z_original = gamma2z(Gammas); % Original source Z for OPA
Z_Cap = 1/(1i*2*pi*Freq(Freq_53MHZ_Index)*InputCap); %input parasitc cap's impedance
R_withCap = 1 ./ ( (1./Z_original) + (1./Z_Cap) ); 
%this is the equivelent impedance, paralleled from orignal source Z and input
%cap
Rs = real( R_withCap ); %take the real part, only resistive part
[NFs_1H] = NF_Calculation_LMH6629_NonInverting(Rf,Rg,Rs);

Dmax = 3;% plot on smith
Dmin = 0;
figure
SmithPlot(Gammas,NFs_1H,Cres,Dmax,Dmin,"Higher",markersize)
title({ ['23Na OPA-NF vs Matched-to impedance, threshold at ', num2str(Dmax), ' dB'], ' ' })
colorbar('ticks', [0,0.25,0.5,0.75,1], 'TickLabels', ...
    { [ num2str(min(NFs_1H)) 'dB' ],  [ num2str( (min(NFs_1H)+((min(NFs_1H)+Dmax)/2))/2 ) 'dB' ],...
    [ num2str( ((min(NFs_1H)+Dmax)/2) ) 'dB'],[ num2str( (((min(NFs_1H)+Dmax)/2)+Dmax)/2 ) 'dB' ],...
    ['>' num2str(Dmax) 'dB'] })


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%


Z_final_OPA_See_2H = Z_right(Freq_30MHZ_Index);
Z_final_OPA_See_23Na = Z_right(Freq_53MHZ_Index);

Z_final_Coil_See_2H = Z_left(Freq_30MHZ_Index);
Z_final_Coil_See_23Na = Z_left(Freq_53MHZ_Index);


Z_original = (Z_final_OPA_See_2H); % ideal source Z for OPA
Z_Cap = 1/(1i*2*pi*Freq(Freq_30MHZ_Index)*InputCap); %input parasitc cap
R_withCap = 1 ./ ( (1./Z_original) + (1./Z_Cap) );
Rs = real( R_withCap ); %
[NFs] = NF_Calculation_LMH6629_NonInverting(Rf,Rg,Rs);
NF_2H = NFs

Z_original = (Z_final_OPA_See_23Na); % ideal source Z for OPA
Z_Cap = 1/(1i*2*pi*Freq(Freq_53MHZ_Index)*InputCap); %input parasitc cap
R_withCap = 1 ./ ( (1./Z_original) + (1./Z_Cap) );
Rs = real( R_withCap ); %
[NFs] = NF_Calculation_LMH6629_NonInverting(Rf,Rg,Rs);
NF_23Na = NFs


%% Calculate Decoupling for 1H
Decoupling_2H = Decoupling_Calculation( Z_coil(Freq_30MHZ_Index),z2gamma( Z_final_Coil_See_2H ))

%% Calculate Decoupling for 23Na
Decoupling_23Na = Decoupling_Calculation( Z_coil(Freq_53MHZ_Index),z2gamma( Z_final_Coil_See_23Na))


%% timer
toc

